@@ -1,13 +1,10 @@
-module Hint.Parsers
+module Hint.Parsers where
 
-where
-
-import Prelude hiding(span)
+import Prelude hiding (span)
 
 import Hint.Base
-import qualified Hint.Compat as Compat
 
-import Control.Monad.Trans ( liftIO )
+import Control.Monad.Trans (liftIO)
 
 import qualified Hint.GHC as GHC
 
@@ -23,11 +20,11 @@ runParser :: MonadInterpreter m => GHC.P a -> String -> m ParseResult
 runParser parser expr =
     do dyn_fl <- runGhc GHC.getSessionDynFlags
        --
-       buf <- Compat.stringToStringBuffer expr
+       buf <- (return . GHC.stringToStringBuffer) expr
        --
        -- ghc >= 7 panics if noSrcLoc is given
-       let srcLoc = Compat.mkSrcLoc (GHC.fsLit "<hint>") 1 1
-       let parse_res = GHC.unP parser (Compat.mkPState dyn_fl buf srcLoc)
+       let srcLoc = GHC.mkRealSrcLoc (GHC.fsLit "<hint>") 1 1
+       let parse_res = GHC.unP parser (GHC.mkPState dyn_fl buf srcLoc)
        --
        case parse_res of
            GHC.POk{}            -> return ParseOk
@@ -50,15 +47,9 @@ failOnParseError parser expr = mayFail go
                           do -- parsing failed, so we report it just as all
                              -- other errors get reported....
                              logger <- fromSession ghcErrLogger
-#if __GLASGOW_HASKELL__ >= 706
                              dflags <- runGhc GHC.getSessionDynFlags
                              let logger'  = logger dflags
                                  errStyle = GHC.defaultErrStyle dflags
-#else
-                             let logger'  = logger
-                                 errStyle = GHC.defaultErrStyle
-
-#endif
                              liftIO $ logger' GHC.SevError
                                               span
                                               errStyle
